@@ -18,6 +18,7 @@ from Cryptodome.Signature import PKCS1_v1_5
 from .compat import decodebytes, encodebytes, quote_plus, urlopen
 from .exceptions import AliPayException, AliPayValidationError
 from .utils import AliPayConfig
+from .loggers import logger
 
 
 # 常见加密算法
@@ -59,6 +60,7 @@ class BaseAliPay:
         alipay_public_key_string=None,
         sign_type="RSA2",
         debug=False,
+        verbose=False,
         config=None
     ):
         """
@@ -73,6 +75,7 @@ class BaseAliPay:
         self._app_notify_url = app_notify_url
         self._app_private_key_string = app_private_key_string
         self._alipay_public_key_string = alipay_public_key_string
+        self._verbose = verbose
         self._config = config or AliPayConfig()
 
         self._app_private_key = None
@@ -135,9 +138,7 @@ class BaseAliPay:
                 data[k] = json.dumps(v, separators=(',', ':'))
         return sorted(data.items())
 
-    def build_body(
-        self, method, biz_content, return_url=None, notify_url=None, append_auth_token=False
-    ):
+    def build_body(self, method, biz_content, **kwargs):
         data = {
             "app_id": self._appid,
             "method": method,
@@ -147,22 +148,21 @@ class BaseAliPay:
             "version": "1.0",
             "biz_content": biz_content
         }
-        if append_auth_token:
-            data["app_auth_token"] = self.app_auth_token
-
-        if return_url is not None:
-            data["return_url"] = return_url
+        data.update(kwargs)
 
         if method in (
-            "alipay.trade.app.pay", "alipay.trade.wap.pay", "alipay.trade.page.pay",
-            "alipay.trade.pay", "alipay.trade.precreate"
-        ) and (notify_url or self._app_notify_url):
-            data["notify_url"] = notify_url or self._app_notify_url
+            "alipay.trade.app.pay", "alipay.trade.wap.pay",
+            "alipay.trade.page.pay", "alipay.trade.pay",
+            "alipay.trade.precreate"
+        ) and not data.get("notify_url") and self._app_notify_url:
+            data["notify_url"] = self._app_notify_url
 
+        if self._verbose:
+            logger.debug("data to be signed")
+            logger.debug(data)
         return data
 
     def sign_data(self, data):
-        data.pop("sign", None)
         # 排序后的字符串
         ordered_items = self._ordered_data(data)
         raw_string = "&".join("{}={}".format(k, v) for k, v in ordered_items)
@@ -171,6 +171,9 @@ class BaseAliPay:
 
         # 获得最终的订单信息字符串
         signed_string = "&".join("{}={}".format(k, quote_plus(v)) for k, v in unquoted_items)
+        if self._verbose:
+            logger.debug("signed srtring")
+            logger.debug(signed_string)
         return signed_string
 
     def _verify(self, raw_content, signature):
@@ -194,15 +197,22 @@ class BaseAliPay:
         message = "&".join(u"{}={}".format(k, v) for k, v in unsigned_items)
         return self._verify(message, signature)
 
-    def api(self, api_name, **kwargs):
+    def client_api(self, api_name, biz_content, **kwargs):
         """
-        alipay.api("alipay.trade.page.pay", **kwargs) ==> alipay.api_alipay_trade_page_pay(**kwargs)
+        alipay api without http request
         """
-        api_name = api_name.replace(".", "_")
-        key = "api_" + api_name
-        if hasattr(self, key):
-            return getattr(self, key)
-        raise AttributeError("Unknown attribute" + api_name)
+        data = self.build_body(api_name, biz_content, **kwargs)
+        return self.sign_data(data)
+
+    def server_api(self, api_name, biz_content, **kwargs):
+        """
+        alipay api with http request
+        """
+        data = self.build_body(api_name, biz_content, **kwargs)
+        # alipay.trade.query => alipay_trade_query_response
+        response_type = api_name.replace(".", "_") + "_response"
+        print(data)
+        return self.verified_sync_response(data, response_type)
 
     def api_alipay_trade_wap_pay(
         self, subject, out_trade_no, total_amount,
@@ -669,6 +679,9 @@ class DCAliPay(BaseAliPay):
         data = super().build_body(*args, **kwargs)
         data["app_cert_sn"] = self.app_cert_sn
         data["alipay_root_cert_sn"] = self.alipay_root_cert_sn
+        if self._verbose:
+            logger.debug("data to be signed")
+            logger.debug(data)
         return data
 
     def load_alipay_public_key_string(self):
@@ -798,6 +811,16 @@ class ISVAliPay(BaseAliPay):
                 raise Exception(msg.format(self._app_auth_code))
         return self._app_auth_token
 
+    def build_body(self, *args, **kwargs):
+        data = super().build_body(*args, **kwargs)
+        if self._app_auth_token:
+            data["app_auth_token"] = self._app_auth_token
+        if self._verbose:
+            logger.debug("data to be signed")
+            logger.debug(data)
+                
+        return data
+
     def api_alipay_open_auth_token_app(self, refresh_token=None):
         """
         response = {
@@ -825,7 +848,6 @@ class ISVAliPay(BaseAliPay):
         data = self.build_body(
             "alipay.open.auth.token.app",
             biz_content,
-            append_auth_token=False
         )
         response_type = "alipay_open_auth_token_app_response"
         return self.verified_sync_response(data, response_type)
@@ -835,7 +857,6 @@ class ISVAliPay(BaseAliPay):
         data = self.build_body(
             "alipay.open.auth.token.app.query",
             biz_content,
-            append_auth_token=False
         )
         response_type = "alipay_open_auth_token_app_query_response"
         return self.verified_sync_response(data, response_type)
